@@ -21,6 +21,9 @@ set pkgdatadir "C:\\Program Files\\BookView"
 # srcdir
 set srcdir [file dirname $argv0]
 
+# error_message
+set error_message ""
+
 # Files to be installed.
 set install_files {
     "bookview/bookview.tcl"
@@ -29,10 +32,12 @@ set install_files {
     "bookview/ndtp.tcl"
     "bookview/help.tcl"
     "bookview/setup.tcl"
-    "bookview/ja.mo"
+    "bookview/ja.mo.win32"
     "bitmap/arrow.xbm"
+    "bitmap/backward.xbm"
     "bitmap/copyright.gif"
     "bitmap/find.gif"
+    "bitmap/forward.xbm"
     "bitmap/menu.gif"
     "bitmap/next.gif"
     "bitmap/prev.gif"
@@ -89,39 +94,36 @@ proc make_main_window {} {
 }
 
 #
-# Install.
+# Generate `bookview/bookview.tcl' from `bookview/bookview.in'.
 #
-proc install_bookview {} {
+proc generate_bookview_tcl {} {
+    global srcdir
     global version
     global pkgdatadir
-    global srcdir
-    global install_files
+    global error_message
 
-    #
-    # Generate `bookview/bookview.tcl' from `bookview/bookview.in'.
-    #
     file delete -force "$srcdir/bookview/bookview.tcl"
     if {[catch {set rfile [open "$srcdir/bookview/bookview.in" "r"]} \
 	    error_message]} {
-	return $error_message
+	return 0
     }
     if {[catch {set wfile [open "$srcdir/bookview/bookview.tcl" "w"]} \
 	    error_message]} {
 	catch {close $rfile}
-	return $error_message
+	return 0
     }
     while {1} {
 	if {[catch {gets $rfile line} error_message]} {
 	    catch {close $rfile}
 	    catch {close $wfile}
-	    return $error_message
+	    return 0
 	}
 	regsub -all {@VERSION@} $line $version line
 	regsub -all {@pkgdatadir@} $line $pkgdatadir line
 	if {[catch {puts $wfile $line} error_message]} {
 	    catch {close $rfile}
 	    catch {close $wfile}
-	    return $error_message
+	    return 0
 	}
 	if {[eof $rfile]} {
 	    break
@@ -130,26 +132,86 @@ proc install_bookview {} {
     catch {close $rfile}
     catch {close $wfile}
 
-    #
-    # Make the directory $pkgdatadir.
-    #
+    return 1
+}
+
+#
+# Generate `bookview/ja.mo.win32' from `bookview/ja.mo'.
+#
+proc generate_ja_mo_win32 {} {
+    global srcdir
+    global version
+    global pkgdatadir
+    global error_message
+
+    file delete -force "$srcdir/bookview/ja.mo.win"
+    if {[catch {set rfile [open "$srcdir/bookview/ja.mo" "r"]} \
+	    error_message]} {
+	return 0
+    }
+    fconfigure $rfile -encoding euc-jp
+
+    if {[catch {set wfile [open "$srcdir/bookview/ja.mo.win32" "w"]} \
+	    error_message]} {
+	catch {close $rfile}
+	return 0
+    }
+    
+    while {1} {
+	if {[catch {gets $rfile line} error_message]} {
+	    catch {close $rfile}
+	    catch {close $wfile}
+	    return 0
+	}
+	if {[catch {puts $wfile $line} error_message]} {
+	    catch {close $rfile}
+	    catch {close $wfile}
+	    return 0
+	}
+	if {[eof $rfile]} {
+	    break
+	}
+    }
+    catch {close $rfile}
+    catch {close $wfile}
+
+    return 1
+}
+
+#
+# Make the directory $pkgdatadir.
+#
+proc create_pkgdatadir {} {
+    global pkgdatadir
+    global error_message
+
     if {![file isdirectory $pkgdatadir]} {
 	if {[catch {file mkdir $pkgdatadir} error_message]} {
-	    return $error_message
+	    return 0
 	}
     }
 
-    #
-    # Install files.
-    #
+    return 1
+}
+
+#
+# Install files.
+#
+proc install_files {} {
+    global srcdir
+    global pkgdatadir
+    global install_files
+    global error_message
+
     foreach src $install_files {
 	set dst "$pkgdatadir/[file tail $src]"
+	regsub {\.win32$} $dst {} dst
 	if {[catch {file copy -force $srcdir/$src $dst} error_message]} {
-	    return $error_message
+	    return 0
 	}
     }
 
-    return "Done."
+    return 1
 }
 
 #
@@ -191,7 +253,29 @@ proc make_result_window {message} {
     destroy .
 }
 
+#
+# Main.
+#
 make_main_window
 regsub -all "\\\\" $pkgdatadir "/" pkgdatadir
-set message [install_bookview]
-make_result_window $message
+
+if {![generate_bookview_tcl]} {
+    make_result_window $error_message
+    exit 1
+}
+if {![generate_ja_mo_win32]} {
+    make_result_window $error_message
+    exit 1
+}
+if {![create_pkgdatadir]} {
+    make_result_window $error_message
+    exit 1
+}
+if {![install_files]} {
+    make_result_window $error_message
+    exit 1
+}
+
+make_result_window "Done."
+exit 0
+
